"""
m_stats: A statistics and probability library for Python
=======================================================

"""
from m_stats import anova
from m_stats import distributions
from m_stats import generals
from m_stats import hypothesis
from m_stats import intervals
from m_stats import categorical
from m_stats import linregr
from m_stats import graph
from m_stats import multiregr
from m_stats import timeseries



# -*- coding: utf-8 -*-
"""
Created on Sun Mar 24 11:59:54 2019

@author: Miguel Ángel Vélez
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches

from m_stats import linregr as lr

#------------------------------------------------------------------------------------------------#
#------------------------------------------ Graph -----------------------------------------------#
#------------------------------------------------------------------------------------------------#

def discrete(distr):
    """
    """
    x_values = np.arange(distr.ppf(0.01), distr.ppf(0.99))
    y_values = distr.pmf(x_values)
    
    plt.plot(x_values, y_values)
    plt.show()
    
def discrete_range(distr, a, b):
    """
    """
    x_values = np.arange(distr.ppf(0.01), distr.ppf(0.99))
    x_values2 = np.arange(a, (b + 1))
    y_values = distr.pmf(x_values)
    y_values2 = distr.pmf(x_values2)
    
    plt.plot(x_values, y_values)
    plt.vlines(x_values2, 0, y_values2, colors='b', lw=5, alpha=0.5)
    plt.show()
    
def continuous(distr):
    """
    """
    x_values = np.linspace(distr.ppf(0.01), distr.ppf(0.99), 100)
    y_values = distr.pdf(x_values)
    
    plt.plot(x_values, y_values)
    plt.show()
    
def continuous_range(distr, a, b):
    """
    """
    x_values = np.linspace(distr.ppf(0.01), distr.ppf(0.99), 100)
    x_values2 = np.linspace(a, b)
    y_values = distr.pdf(x_values)
    y_values2 = distr.pdf(x_values2)
    
    plt.plot(x_values, y_values)
    plt.vlines(x_values2, 0, y_values2, colors='b', lw=5, alpha=0.5)
    plt.show()
    
def hypothesis(distr, test, sign, tail):
    """
    
    Parameters:
    --------------------------
    
    Returns:
    --------------------------
    """
    alpha = 1 - sign
    x_values = np.linspace(distr.ppf(0.001), distr.ppf(0.999), 100)
    y_values = distr.pdf(x_values)    
    plt.plot(x_values, y_values, 'black')
    
    if (tail == 'two'):
        crit = distr.interval(alpha)
        x_values2 = np.append(np.linspace(distr.ppf(0.001), crit[0]), 
                              np.linspace(crit[1], distr.ppf(0.999)))
        y_values2 = distr.pdf(x_values2)
        
        x_values3 = np.linspace(crit[0] + 0.1, crit[1] - 0.1)
        y_values3 = distr.pdf(x_values3)
        
        plt.vlines(crit[0], 0, distr.pdf(crit[0]), colors='black', lw=2, alpha=1)
        plt.vlines(crit[1], 0, distr.pdf(crit[1]), colors='black', lw=2, alpha=1)
        
        plt.vlines(x_values2, 0, y_values2, colors='gray', lw=5, alpha=0.5)
        plt.vlines(x_values3, 0, y_values3, colors='khaki', lw=5, alpha=0.5)
        
        plt.annotate(
                "Critical value: {}".format(round(crit[0], 2)),
                xy=(crit[0], distr.pdf(crit[0])),
                xytext=(50, 30),
                textcoords='offset points', ha='right', va='bottom',
                bbox=dict(boxstyle='round,pad=0.5', fc='darkred', alpha=0.5),
                arrowprops=dict(arrowstyle = '->', connectionstyle='arc3,rad=0')
                )
        plt.annotate(
                "Critical value: {}".format(round(crit[1], 2)),
                xy=(crit[1], distr.pdf(crit[1])),
                xytext=(50, 30),
                textcoords='offset points', ha='right', va='bottom',
                bbox=dict(boxstyle='round,pad=0.5', fc='darkred', alpha=0.5),
                arrowprops=dict(arrowstyle = '->', connectionstyle='arc3,rad=0')
                )
        
    if (tail == 'left'):
        crit = distr.ppf(1 - alpha)
        
        x_values2 = np.linspace(distr.ppf(0.001), crit)
        y_values2 = distr.pdf(x_values2)
        
        x_values3 = np.linspace(crit + 0.1, distr.ppf(0.999))
        y_values3 = distr.pdf(x_values3)
        
        plt.vlines(x_values2, 0, y_values2, colors='gray', lw=5, alpha=0.5)
        plt.vlines(x_values3, 0, y_values3, colors='khaki', lw=5, alpha=0.5)
        
        plt.annotate(
            "Critical value: {}".format(round(crit, 2)),
            xy=(crit, distr.pdf(crit)),
            xytext=(50, 30),
            textcoords='offset points', ha='right', va='bottom',
            bbox=dict(boxstyle='round,pad=0.5', fc='darkred', alpha=0.5),
            arrowprops=dict(arrowstyle = '->', connectionstyle='arc3,rad=0')
            )
        
    if (tail == 'right'):
        crit = distr.ppf(alpha)
        
        x_values2 = np.linspace(crit, distr.ppf(0.999))
        y_values2 = distr.pdf(x_values2)
        
        x_values3 = np.linspace(distr.ppf(0.001), crit - 0.1)
        y_values3 = distr.pdf(x_values3)
        
        plt.vlines(x_values2, 0, y_values2, colors='gray', lw=5, alpha=0.5)
        plt.vlines(x_values3, 0, y_values3, colors='khaki', lw=5, alpha=0.5)
        
        plt.annotate(
            "Critical value: {}".format(round(crit, 2)),
            xy=(crit, distr.pdf(crit)),
            xytext=(50, 30),
            textcoords='offset points', ha='right', va='bottom',
            bbox=dict(boxstyle='round,pad=0.5', fc='darkred', alpha=0.5),
            arrowprops=dict(arrowstyle = '->', connectionstyle='arc3,rad=0')
            )
            
            
            
        
    plt.vlines(test, 0, distr.pdf(test), colors='darkred', lw=2, alpha=1)
    plt.annotate(
            "Statistical test: {}".format(round(test, 2)),
            xy=(test, distr.pdf(test)),
            xytext=(60, 80),
            textcoords='offset points', ha='right', va='bottom',
            bbox=dict(boxstyle='round,pad=0.5', fc='yellow', alpha=0.5),
            arrowprops=dict(arrowstyle = '->', connectionstyle='arc3,rad=0')
            )
    
    plt.subplots_adjust(left=0.125*2, bottom=None, right=0.9*2, top=None,
            wspace=None, hspace=None)
    
    gray_patch = mpatches.Patch(color='gray', label='Reject zone')
    khaki_patch = mpatches.Patch(color='#F7F2C5', label='No reject zone')
  
    plt.legend(bbox_to_anchor=(1.01, 1), loc=2, borderaxespad=0.,
               handles=[gray_patch, khaki_patch])
               
    
    plt.show()
    
    
def linregr(x, y):
    """
    """
    plt.scatter(x, y, color='seagreen')
    fx = [(lr.regression(x, y))(i) for i in x]
    plt.plot(x, fx, color='dimgray')
    plt.title(r'$\hat Y = {} + {}x$'.format(round(lr.intercept(x, y), 2), round(lr.slope(x, y), 2)),
              fontsize=17)

    
    plt.show()
    
def trendline(y, t):
    """
    """
    plt.scatter(t, y, color='lightblue')
    ft = [(lr.regression(t, y))(i) for i in t]
    plt.plot(t, ft, color='dimgray')
    
    plt.show()
    

def timeserie(y, t):
    """
    """
    plt.scatter(t, y)
    plt.plot(t, y)
    
    plt.ylabel("Y values")
    plt.xlabel("Time values")
    
    plt.title("Time Serie Graph")
    
    plt.show()

#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#


# -*- coding: utf-8 -*-
"""
Created on Mon Mar 11 10:36:13 2019

@author: Miguel Angel Velez
"""

#------------------------------------------------------------------------------------------------#
#------------------------------------------ Generals --------------------------------------------#
#------------------------------------------------------------------------------------------------#


def get_z(x, mu, sd, **kwargs):
    """
    Function to get the Z value (standard deviations number under or over the average)
    correspondent of the x value (parameter) in the standard normal distribution.
    
    Parameters:
    --------------------------
    x : double, float
        Specific value from the random variable.
    
    mu : double, float
        Population average given.
        
    sd : double, float
        Population or sample standard deviation given.
        
    n : int, optional
        Sample size, if you are working with sample distributions. (default=None)
        
    Returns:
    --------------------------
    z : float, double
        The Z value (standard deviations number under or over the average)
        correspondent of the x value (parameter) in the standard normal distribution.
    """
    if not kwargs:
        z = ((x - mu) / sd)
        return z
    else:
        n = kwargs.get('n', None)
        if (n <= 30):
            print("The sample size must be greater than 30.")
        else:
            z = ((x - mu) / (sd / n**0.5))
            return z
        

def get_z_2p(x_1, x_2, s1, s2, n1, n2):
    """
    
    """
    if (n1 <= 30 or n2 <= 30):
        print("The sample sizes must be greater than 30.")
    else:
        s_error = ((s1**2 / n1) + (s2**2 / n2))**0.5
        
        z = (x_1 - x_2) / s_error
        return z
    

def get_t(x, mu, s, n):
    """
    Function to get the T value correspondent of the X value in the t-student
    distribution.
    
    Parameters:
    --------------------------
    x : double, float
        Specific value from the random variable.
    
    mu : double, float
        Population average given.
        
    sd : double, float
        Sample standard deviation given.
        
    n : int, optional
        Sample size, less than 30.
        
    Returns:
    --------------------------
    t : float, double
        The T value correspondent of the X value in the t-student.
    """
    if n > 30:
        print("The sample size must be less than 30.")
    else:
        t = (x - mu) / (s / (n**0.5))
        return t
    
    
def get_t_2p(x_1, x_2, s1, s2, n1, n2, var):
    """
    """
    if (n1 > 30 or n2 > 30):
        print("The sample sizes must be less than 30.")
    else:
        if (var is True):
            df = n1 + n2 - 2
            s2p = ((s1**2 * (n1 - 1)) + ((s2**2) * (n2 - 1))) / df
            s2p_error = ((s2p/n1) + (s2p/n2))**0.5
            
            t = (x_1 - x_2) / s2p_error
            return t
        else:
            s2p_error = ((s1**2 / n1) + (s2**2 / n2))**0.5
            t = (x_1 - x_2) / s2p_error
            return t
            
            

    
def get_z_prop(p, pi, n):
    """
    Function to get the Z value (standard deviations number under or over the average)
    correspondent of the x value (parameter) in the standard normal distribution.
    Applied in proportions.
    
    Parameters:
    --------------------------
    p : double, float
        "Succeses" observations sample proportion.
        Each value should be in the range [0, 1].
    
    pi : double, float
        Population proportion given.
        Each value should be in the range [0, 1].
        
    n : int
        Sample size.
        
    Returns:
    --------------------------
    z : float, double
        The Z value (standard deviations number under or over the average)
        correspondent of the x value (parameter) in the standard normal distribution.
    """
    error = ((pi * (1 - pi)) / n)**0.5
    z = (p - pi) / error
    return z

def get_z_2prop(p1, p2, n1, n2):
    """
    """
    sp_error = (((p1 * (1 - p1)) / n1) + ((p2 * (1 - p2)) / n2))**0.5
    
    z = (p1 - p2) / sp_error
    return z


def get_df_var(n1, n2, s1, s2):
    """
    """
    df = ((((s1**2) / n1) + ((s2**2) / n2))**2) / (((((s1**2) / n1)**2) / 
            (n1 - 1)) + ((((s2**2) / n2)**2) / (n2 - 1)))
    return df


def get_d_pair(n, cond1, cond2):
    """
    """
    d1 = [cond1[x] - cond2[x] for x in range(n)]
    d21 = [x**2 for x in d1]
    sumd1 = sum(d1)
    sumd21 = sum(d21)
    d_ = sumd1 / n
    sd = ((sumd21 - (n * (d_**2))) / (n - 1))**0.5
    sd_error = sd / (n**0.5)
    
    d = d_ / sd_error
    return d

def get_f_2p(var1, var2):
    """
    """
    f = var1 / var2
    return f

#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#


# -*- coding: utf-8 -*-
"""
Created on Sat Apr 20 15:18:27 2019

@author: Miguel Ángel Vélez

"""

import numpy as np

#------------------------------------------------------------------------------------------------#
#------------------------------------- Multiple Regression --------------------------------------#
#------------------------------------------------------------------------------------------------#

#y1 = [15, 17, 13, 23, 16, 21, 14, 20, 24, 17, 16, 18, 23, 15, 16]
##
#x1 = [10, 12, 8, 17, 10, 15, 10, 14, 19, 10, 11, 13, 16, 10, 12]
#x2 = [2.4, 2.72, 2.08, 3.68, 2.56, 3.36, 2.24, 3.2, 3.84, 2.72, 2.07, 2.33, 2.98, 1.94, 2.17]

def beta_hat(y, *args):
    """
    """
    x = list(args)
    
    ones = [1] * (len(y))
    X = np.matrix([ones] + x).transpose()
    
    #print(np.dot(transpose_X, X))
    # print(xs * transpose_x)
    
    #print(
            #np.dot(np.linalg.inv(np.dot(X.transpose(), X)), np.dot(y, X.transpose()))
            #np.dot(y, np.dot(np.linalg.inv(np.dot(X.transpose(), X)), X.transpose()))
            #np.dot(np.dot(np.linalg.inv(np.dot(X.transpose(), X)), X.transpose()), y).tolist()[0]
            #)
            
    beta_hat = np.dot(np.dot(np.linalg.inv(np.dot(X.transpose(), X)), X.transpose()), y).tolist()[0]
    
    return beta_hat

def regression(y, *args):
    """
    """
    betas = beta_hat(y, *args)
    model = lambda *xs : betas[0] + sum([(betas[i] * xs[i - 1]) for i in range(1, len(betas))])
    model.shape = "ÿ = {}".format(round(betas[0], 2))
    for i in range(1, len(betas)):
        model.shape += (" + {}X{}".format(round(betas[i], 2), i))
    return model

def y_hat(y, *args):
    """
    """
    model = regression(y, *args)
    
    length_args = len(args[0])
    length_parent = len(args)
    
    y_hat = []
    
    for i in range(0, length_args):
        aux = []
        #print("aux: ", aux)
        for j in range(0, length_parent):
            #print("args: ", args[j][i])
            aux.append(args[j][i])
            
        #print("*aux: ", *aux)
        y_hat.append(model(*aux))
        #print("yhat: ", y_hat)
        
    return y_hat
    

#print(y_hat(y1, x1, x2))

def get_se(y, *args):
    """
    """
    y_hats = y_hat(y, *args)
    
    n = len(y)
    k = len(args)
    
    se = (sum([(y[i] - y_hats[i])**2 for i in range(n)]) / (n - k - 1))**0.5
    return se

#print(get_se(y1, x1, x2))

def get_scr(y, *args):
    """
    """
    y_hats = y_hat(y, *args)
    n = len(y)
    mean_y = np.mean(y)
    
    scr = sum([(y_hats[i] - mean_y)**2 for i in range(n)])
    return scr

def get_sct(y, *args):
    """
    """
    mean_y = np.mean(y)
    n = len(y)
    sct = sum([(y[i] - mean_y)**2 for i in range(n)])
    return sct

def get_sce(y, *args):
    """
    """
    y_hats = y_hat(y, *args)
    n = len(y)
    
    sce = sum([(y[i] - y_hats[i])**2 for i in range(n)])
    return sce

def get_r2(y, *args):
    """
    """
    r2 = get_scr(y, *args) / get_sct(y, *args)
    return r2

def get_r2_(y, *args):
    """
    """
    n = len(y)
    k = len(args)
    
    r2_ = 1 - (1 - get_r2(y, *args)) * ((n - 1) / (n - k -1))
    return r2_

def get_cmr(y, *args):
    """
    """
    k = len(args)
    cmr = get_scr(y, *args) / k
    return cmr

def get_cme(y, *args):
    """
    """
    n = len(y)
    k = len(args)
    cme = get_sce(y, *args) / (n - k - 1)
    return cme

def get_fratio(y, *args):
    """
    """
    fratio = get_cmr(y, *args) / get_cme(y, *args)
    return fratio

def matrix_corr(y, *args):
    """
    """
    matrix = np.corrcoef(y, args)
    return matrix

def get_sr(rij, n):
    """
    """
    sr = ((1 - (rij)**2) / (n - 2))**0.5
    return sr

def t_multicollinearity(rij, sr):
    """
    """
    t = rij / sr
    return t

def t_regression(b, beta, sb):
    """
    """
    t = (b - beta) / sb
    return t

def get_sbs(y, *x):
    """
    """
    xs = list(x)
    ones = [1] * len(y)
    X = np.matrix([ones] + xs).transpose()
    
    X_matrix = np.linalg.inv(np.dot(X.transpose(), X))
    
    cme = get_cme(y, *x)
    
    C = cme * X_matrix
    
    sbs = [(C[i, i]**0.5) for i in range(len(x) + 1)]
    
    return sbs
    
#print(get_sbs(y1, x1, x2))

#print(matrix_corr(y1, x1, x2))
#print(get_sr(matrix_corr(y1, x1, x2)[1, 2], len(y1)))

#------------------------------------------------------------------------------------------------#



# -*- coding: utf-8 -*-
"""
Created on Mon Mar 11 10:36:13 2019

@author: Miguel Angel Velez
"""

import scipy.stats as ss
from m_stats import generals, linregr

#------------------------------------------------------------------------------------------------#
#------------------------------------- Confidence Intervals -------------------------------------#
#------------------------------------------------------------------------------------------------#

def norm(n, x_, sd, alpha):
    """
    Function to get the confidence interval to estimate the average population
    when we have large samples (n >= 30).
    
    Parameters:
    --------------------------
    n : int
        Sample size.
        
    x_ : float, double
        Sample average.
    
    sd : float, double
        Sample or population standard deviation.
        
    alpha : float, double
        Confidence level to estimate our unknown parameter (the average population).
        Each value should be in the range [0, 1].
    
    Returns:
    --------------------------
    interval : tuple
        The confidence interval, with the minimum value at [0] and the maximum value
        at [1] for the unknown parameter (the average population).
    """
    if (n <= 30):
        print("The sample size must be greater than 30.")
    else:
        sd_error = sd / (n**0.5)
        z_value = ss.norm.interval(alpha)[1]
        
        minimum = x_ - (z_value * sd_error)
        maximum = x_ + (z_value * sd_error)
        
        return minimum, maximum
    
def norm_2p(n1, n2, x_1, x_2, sd1, sd2, alpha):
    """
    Function to get the confidence interval to estimate the difference between
    2 population averages in large samples (n >= 30).
    
    Parameters:
    --------------------------
    n1 : int
        Sample 1 size.
        
    n2 : int
        Sample 2 size.
        
    x_1 : float, double
        Sample 1 average.
    
    x_2 : float, double
        Sample 2 average.
    
    sd1 : float, double
        Sample or population 1 standard deviation.
        
    sd2 : float, double
        Sample or population 2 standard deviation.
        
    alpha : float, double
        Confidence level to estimate our unknown parameter (the difference between
        2 population averages).
        Each value should be in the range [0, 1].
    
    Returns:
    --------------------------
    interval : tuple
        The confidence interval, with the minimum value at [0] and the maximum value
        at [1] for the unknown parameter (the difference between
        2 population averages).
    """
    if (n1 <= 30 or n2 <= 30):
        print("The sample sizes must be greater than 30.")
    else:
        sd_error = ((sd1**2 / n1) + (sd2**2 / n2))**0.5
        z_value = ss.norm.interval(alpha)[1]
        
        minimum = (x_1 - x_2) - (z_value * sd_error)
        maximum = (x_1 - x_2) + (z_value * sd_error)
        
        return minimum, maximum

           
def t(n, x_, s, alpha):
    """
    Function to get the confidence interval to estimate the average population
    when we have small samples (n < 30).
    
    Parameters:
    --------------------------
    n : int
        Sample size.
        
    x_ : float, double
        Sample average.
    
    s : float, double
        Sample standard deviation.
        
    alpha : float, double
        Confidence level to estimate our unknown parameter (the average population).
        Each value should be in the range [0, 1].
    
    Returns:
    --------------------------
    interval : tuple
        The confidence interval, with the minimum value at [0] and the maximum value
        at [1] for the unknown parameter (the average population).
    """    
    if (n > 30):
        print("The sample size must be less than 30.")
    else:
        df = n - 1
        s_error = s / (n**0.5)
        t_value = ss.t.interval(alpha, df)[1]
        
        minimum = x_ - (t_value * s_error)
        maximum = x_ + (t_value * s_error)
        
        return minimum, maximum

def t_2p(n1, n2, x_1, x_2, s1, s2, alpha, var):
    """
    Function to get the confidence interval to estimate the difference between
    2 population averages in small samples (n < 30).
    
    Parameters:
    --------------------------
    n1 : int
        Sample 1 size.
        
    n2 : int
        Sample 2 size.
        
    x_1 : float, double
        Sample 1 average.
    
    x_2 : float, double
        Sample 2 average.
    
    s1 : float, double
        Sample 1 standard deviation.
        
    s2 : float, double
        Sample 2 standard deviation.
        
    alpha : float, double
        Confidence level to estimate our unknown parameter (the difference between
        2 population averages).
        Each value should be in the range [0, 1].
        
    var : bool
        Are the population variances equal?
        True if population variances are equal.
        False if population variances are different.
    
    Returns:
    --------------------------
    interval : tuple
        The confidence interval, with the minimum value at [0] and the maximum value
        at [1] for the unknown parameter (the difference between
        2 population averages).
    """    
    if (n1 > 30 or n2 > 30):
        print("The sample sizes must be less than 30.")
    else:
        if (var is True):
            df = n1 + n2 - 2
            s2p = (((s1**2) * (n1 - 1)) + (s2**2 * (n2 - 1))) / df
            sp_error = ((s2p / n1) + (s2p / n2))**0.5
            t_value = ss.t.interval(alpha, df)[1]
            
            minimum = (x_1 - x_2) - (t_value * sp_error)
            maximum = (x_1 - x_2) + (t_value * sp_error)
        
            return minimum, maximum
        else:
            df = generals.get_df_var(n1, n2, s1, s2)
            
            sp_error = (((s1**2) / n1) + ((s2**2) / n2))**0.5
            t_value = ss.t.interval(alpha, df)[1]
            
            minimum = (x_1 - x_2) - (t_value * sp_error)
            maximum = (x_1 - x_2) + (t_value * sp_error)
            
            return minimum, maximum


def prop(n, p, alpha):
    """
    Function to get the confidence interval to estimate the population proportion.
    
    Parameters:
    --------------------------
    n : int
        Sample size.
        
    p : float, double
        Sample proportion.
        Each value should be in the range [0, 1].
            
    alpha : float, double
        Confidence level to estimate our unknown parameter (the population proportion).
        Each value should be in the range [0, 1].
    
    Returns:
    --------------------------
    interval : tuple
        The confidence interval, with the minimum value at [0] and the maximum value
        at [1] for the unknown parameter (the population proportion).
    """   
    sp_error = ((p * (1 - p)) / n)**0.5
    z_value = ss.norm.interval(alpha)[1]
    
    minimum = p - (z_value * sp_error)
    maximum = p + (z_value * sp_error)
    
    return minimum, maximum

def prop_2p(n1, n2, p1, p2, alpha):
    """
    Function to get the confidence interval to estimate the difference between
    2 population proportions.
    
    Parameters:
    --------------------------
    n1 : int
        Sample 1 size.
    
    n2 : int
        Sample 2 size.
        
    p1 : float, double
        Sample 1 proportion.
        Each value should be in the range [0, 1].
        
    p2 : float, double
        Sample 2 proportion.
        Each value should be in the range [0, 1].
            
    alpha : float, double
        Confidence level to estimate our unknown parameter (the population proportion).
        Each value should be in the range [0, 1].
    
    Returns:
    --------------------------
    interval : tuple
        The confidence interval, with the minimum value at [0] and the maximum value
        at [1] for the unknown parameter (the difference between
        2 population proportions).
    """
    sp_error = (((p1 * (1 - p1)) / n1) + ((p2 * (1 - p2)) / n2))**0.5
    z_value = ss.norm.interval(alpha)[1]
    
    minimum = (p1 - p2) - (z_value * sp_error)
    maximum = (p1 - p2) + (z_value * sp_error)
    
    return minimum, maximum
  

def pair_2p(n, cond1, cond2, alpha):
    """
    Function to get the confidence interval to estimate the difference between
    2 population averages in paired samples.
    
    Parameters:
    --------------------------
    n : int
        Sample sizes.
        
    cond1 : list
        Sample initial condition list.
    
    cond2 : list
        Sample final condition list.
        
    alpha : double, float
        Confidence level to estimate our unknown parameter (the difference between
        2 population averages).
        Each value should be in the range [0, 1].
        
    Returns:
    --------------------------
    interval : tuple
        The confidence interval, with the minimum value at [0] and the maximum value
        at [1] for the unknown parameter (the difference between
        2 population averages).
    
    """
    d1 = [cond1[x] - cond2[x] for x in range(n)]
    d21 = [x**2 for x in d1]
    sumd1 = sum(d1)
    sumd21 = sum(d21)
    d_ = sumd1 / n
    sd = ((sumd21 - (n * (d_**2))) / (n - 1))**0.5
    sd_error = sd / (n**0.5)
    
    if n <= 30:
        df = n - 1
        t_value = ss.t.interval(alpha, df)[1]
        
        minimum = d_ - (t_value * sd_error)
        maximum = d_ + (t_value * sd_error)
        
        return minimum, maximum
    
    else:
        z_value = ss.norm.interval(alpha)[1]
        
        minimum = d_ - (z_value * sd_error)
        maximum = d_ + (z_value * sd_error)
        
        return minimum, maximum
    
    

def n_avg(alpha, s, error):
    """
    Function to determine the appropiate sample size to estimate the average population.
    
    Parameters:
    --------------------------         
    alpha : float, double
        Confidence level to determine our sample size.
        Each value should be in the range [0, 1].
        
    s: float, double
        Pilot sample standard deviation, or population standar deviation.
        
    error: float, double
        Tolerable size error.
    
    Returns:
    --------------------------
    n : int
        The appropiate sample size to estimate the average population.
        This sample size is rounded.
    """
    z_value = ss.norm.interval(alpha)[1]
    
    n = ((z_value**2)*(s**2)) / (error**2)
    
    return round(n)

def n_avg_2p(alpha, s1, s2, error):
    """
    Function to determine the appropiate sample size to estimate the difference
    between 2 average populations.
    
    Parameters:
    --------------------------         
    alpha : float, double
        Confidence level to determine our sample size.
        Each value should be in the range [0, 1].
        
    s1 : float, double
        Pilot sample 1 standard deviation, or population standar deviation.
        
    s2 : float, double
        Pilot sample 2 standard deviation, or population standar deviation.
        
    error: float, double
        Tolerable size error.
    
    Returns:
    --------------------------
    n : int
        The appropiate sample size to estimate the he difference
        between 2 average populations.
        This sample size is rounded.
    """
    z_value = ss.norm.interval(alpha)[1]
    
    n = ((z_value**2)*(s1**2 + s2**2)) / (error**2)
    
    return round(n)



def n_prop(alpha, p, error):
    """
    Function to determine the appropiate sample size to estimate the population proportion.
    
    Parameters:
    --------------------------         
    alpha : float, double
        Confidence level to determine our sample size.
        Each value should be in the range [0, 1].
        
    p: float, double
        Sample proportion.
        Each value should be in the range [0, 1].
        
    error: float, double
        Tolerable size error.
    
    Returns:
    --------------------------
    n : int
        The appropiate sample size to estimate the population proportion.
        This sample size is rounded.
    """
    z_value = ss.norm.interval(alpha)[1]
    
    n = ((z_value**2) * (p * (1 - p))) / (error**2)
    
    return round(n)


def n_prop_2p(alpha, p1, p2, error):
    """
    Function to determine the appropiate sample size to estimate the 
    difference between 2 population proportions.
    
    Parameters:
    --------------------------         
    alpha : float, double
        Confidence level to determine our sample size.
        Each value should be in the range [0, 1].
        
    p1: float, double
        Sample proportion 1.
        Each value should be in the range [0, 1].
        
    p2: float, double
        Sample proportion 2.
        Each value should be in the range [0, 1].
        
    error: float, double
        Tolerable size error.
    
    Returns:
    --------------------------
    n : int
        The appropiate sample size to estimate the difference between 
        2 population proportions.
        This sample size is rounded.
    """
    z_value = ss.norm.interval(alpha)[1]
    
    n = ((z_value**2) * ((p1 * (1 - p1)) + ((p2 * (1 - p2))))) / (error**2)
    
    return round(n)
            
def f_2p(n1, n2, var1, var2, alpha):
    """
    """
    df1 = n1 - 1
    df2 = n2 - 1
    f_value = ss.f(df1, df2).interval(alpha)
    
    minimum = var1 / (f_value[1] * var2)
    maximum = var1 / (f_value[0] * var2)
    
    return minimum, maximum

def beta(x, y, alpha):
    """
    """
    df = len(x) - 1
    t_value = ss.t(df).interval(alpha)[1]
    
    minimum = linregr.slope(x, y) - (t_value * linregr.get_sb(x, y))
    maximum = linregr.slope(x, y) + (t_value * linregr.get_sb(x, y))
    
    return minimum, maximum
    

#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#


# -*- coding: utf-8 -*-
"""
Created on Mon Mar 11 10:40:23 2019

@author: Miguel Angel Velez
"""

import scipy.stats as ss

#------------------------------------------------------------------------------------------------#
#------------------------------------- Hypothesis Testing ---------------------------------------#
#------------------------------------------------------------------------------------------------#

def crit_val_norm(sign, tail):
    """
    Function to calculate, with the standard normal distribution, the critical 
    values according to the level of significance and the tail-type test.
    
    Parameters:
    --------------------------
    sign : float, double
        Level of significance of the test.
        Each value should be in the range [0, 1].
        
    tail : string
        Tail-type test, it could be 'left', 'right' or 'two'.
        
    Returns:
    --------------------------
    crit_val : tuple, float
        The critical values or value, according to the level of significance and
        the tail-type test.
    """
    if tail == 'two':
        alpha = 1 - (sign/2)
        crit_val = ss.norm.ppf(1 - alpha), ss.norm.ppf(alpha)
        return crit_val
    
    if tail == 'left':
        alpha = 1 - sign
        crit_val = ss.norm.ppf(1 - alpha)
        return crit_val
    
    if tail == 'right':
        alpha = 1 - sign
        crit_val = ss.norm.ppf(alpha)
        return crit_val
    
    print("You must input a valid tail ('two', 'left' or 'right')")
    

def crit_val_t(df, sign, tail):
    """
    Function to calculate, with the t-student distribution, the critical 
    values according to the level of significance and the tail-type test.
    
    Parameters:
    --------------------------
    df : int
        Degrees of freedom (n - 1). It's the sample size minus 1.
        
    sign : float, double
        Level of significance of the test.
        Each value should be in the range [0, 1].
        
    tail : string
        Tail-type test, it could be 'left', 'right' or 'two'.
        
    Returns:
    --------------------------
    crit_val : tuple, float
        The critical values or value, according to the level of significance and
        the tail-type test.
    """
    if tail == 'two':
        alpha = 1 - (sign/2)
        crit_val = ss.t.ppf(1 - alpha, df), ss.t.ppf(alpha, df)
        return crit_val
    
    if tail == 'left':
        alpha = 1 - sign
        crit_val = ss.t.ppf(1 - alpha, df)
        return crit_val
    
    if tail == 'right':
        alpha = 1 - sign
        crit_val = ss.t.ppf(alpha, df)
        return crit_val
    
    print("You must input a valid tail ('two', 'left' or 'right')")
    

def crit_val_f(df1, df2, sign):
    """
    """
    alpha = 1 - sign
    crit_val = ss.f.ppf(alpha, df1, df2)
    return crit_val

def crit_val_chi2(df, sign):
    """
    """
    alpha = 1 - sign
    crit_val = ss.chi2.ppf(alpha, df)
    return crit_val
            
    
def reject_h0(crit_val, value, tail):
    """
    Function to determine if reject the null hypothesis or not reject it based
    on the tail-type test.
    
    Parameters:
    --------------------------
    crit_val : tuple, float
        Critical values to consider.
        
    value : float, double
        Value to compare with critical values.
        
    tail : string
        Tail-type test, it could be 'left', 'right' or 'two'.
        
    Returns:
    --------------------------
    decision : bool
        True if we should reject null hypothesis,
        False if we shouldn't reject null hypothesis.
    """
    if tail == 'two':
        return (value < crit_val[0] or value > crit_val[1])
    
    if tail == 'left':
        return value < crit_val
    
    if tail == 'right':
        return value > crit_val
    
    print("You must input a valid tail ('two', 'left' or 'right')")
    
    
def get_p(z, tail):
    """
    Function to determine the p-value (minimum significance level to reject
    the null hypothesis).
    
    Parameters:
    --------------------------
    z : double, float
        Z-value of the test statistic.
        
    tail : string
        Tail-type test, it could be 'left', 'right' or 'two'.
        
    Returns:
    --------------------------
    p : double, float
        P-value, the minimum significance level to reject the null hypothesis.
    """
    z_area = ss.norm.cdf(abs(z))
    
    if tail == 'two':
        p = 2 * (1 - z_area)
        return p
    
    else:
        p = 1 - z_area
        return p

            
    
#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#


# -*- coding: utf-8 -*-
"""
Created on Mon Mar 11 11:04:29 2019

@author: Miguel Angel Velez
"""

import numpy as np
import scipy.stats as ss

#------------------------------------------------------------------------------------------------#
#------------------------------------------- ANOVA ----------------------------------------------#
#------------------------------------------------------------------------------------------------#

def get_sct(*args):
    """
    """
    arr_tot = np.concatenate(args)
    big_mean = np.mean(arr_tot)
    sct = sum([(x - big_mean)**2 for x in arr_tot])
    return sct
    
def get_sctr(*args):
    """
    """
    arr_tot = np.concatenate(args)
    big_mean = np.mean(arr_tot)
    sctr = sum([(len(x) * ((np.mean(x) - big_mean)**2)) for x in args])
    return sctr
   
    
def get_sce(*args):
    """
    """        
    sce = 0
    
    for x in args:
        for xj in x:
            sce += (xj - np.mean(x))**2
    return sce

def get_sce2(*args):
    """
    """
    sct = get_sct(*args)
    sctr = get_sctr(*args)
    scbl = get_scbl(*args)
    
    sce = sct - sctr - scbl
    return sce

def get_scbl(*args):
    """
    """
    lists = args
    blocks = list(map(list, zip(*lists)))
    arr_tot = np.concatenate(blocks)
    big_mean = np.mean(arr_tot)
    scbl = sum([(len(x) * ((np.mean(x) - big_mean)**2)) for x in blocks])
    
    return scbl
    

def get_cmt(*args):
    """
    """
    arr_tot = np.concatenate(args)
    df = len(arr_tot) - 1
    sct = get_sct(*args)
    cmt = sct / df
    return cmt

def get_cmtr(*args):
    """
    """
    sctr = get_sctr(*args)
    df = len(args) - 1
    cmtr = sctr / df
    return cmtr

def get_cme(*args):
    """
    """
    arr_tot = np.concatenate(args)
    sce = get_sce(*args)
    df = len(arr_tot) - len(args)
    cme = sce / df
    return cme

def get_cme2(*args):
    """
    """
    sce = get_sce2(*args)
    df = (len(args) - 1) * (len(args[0]) - 1)
    cme = sce / df
    return cme

def get_cmbl(*args):
    """
    """
    scbl = get_scbl(*args)
    df = len(args[0]) - 1
    cmbl = scbl / df
    return cmbl
    
def get_fratio(*args):
    """
    """
    cmtr = get_cmtr(*args)
    cme = get_cme(*args)
    fratio = cmtr / cme
    return fratio

def get_fratio2(*args):
    """
    """
    cmtr = get_cmtr(*args)
    cme = get_cme2(*args)
    fratio = cmtr / cme
    return fratio

def get_fratio2_cmbl(*args):
    """
    """
    cmbl = get_cmbl(*args)
    cme = get_cme2(*args)
    fratio = cmbl / cme
    return fratio

def get_dms(*args, sign):
    """
    """
    cme = get_cme(*args)
    
    
    
    for x in range(len(args)):
        for xj in range(x + 1, len(args)):
            if(len(args[x]) == len(args[xj])):
                fratio = ss.f.ppf((1 - sign), 1, (len(np.concatenate(args)) - len(args)))
                dms = (((1 / len(args[x])) + (1 / len(args[xj]))) * cme * fratio)**0.5
                x_mean = np.mean(args[x])
                xj_mean = np.mean(args[xj])
                x_diff = abs(x_mean - xj_mean)
                if (x_diff > dms):
                    print("|x_{} - x_{}| = |{} - {}| = {} > {}".format(x+1, xj+1,
                          x_mean, xj_mean, x_diff, dms))
                else:
                    print("|x_{} - x_{}| = |{} - {}| = {} <= {}".format(x+1, xj+1,
                          x_mean, xj_mean, x_diff, dms))
            else:
                fratio = ss.f.ppf((1 - sign), (len(args) - 1), (len(np.concatenate(args)) - len(args)))
                dms = (((1 / len(args[x])) + (1 / len(args[xj]))) * cme * fratio)**0.5
                x_mean = np.mean(args[x])
                xj_mean = np.mean(args[xj])
                x_diff = abs(x_mean - xj_mean)
                if (x_diff > dms):
                    print("|x_{} - x_{}| = |{} - {}| = {} > {}".format(x+1, xj+1,
                          x_mean, xj_mean, x_diff, dms))
                else:
                    print("|x_{} - x_{}| = |{} - {}| = {} <= {}".format(x+1, xj+1,
                          x_mean, xj_mean, x_diff, dms))
                        
#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#
#------------------------------------------------------------------------------------------------#




from distutils.core import setup
setup(
  name = 'm_stats',
  packages = ['m_stats'],
  version = '0.1.1',
  license='MIT',
  description = 'A statistics and probability library for Python',
  author = 'Miguel Angel Velez',
  author_email = 'mvelezg99@gmail.com',
  url = 'https://github.com/mvelezg99',
  download_url = 'https://github.com/mvelezg99/m_stats/archive/0.1.1.tar.gz',
  keywords = ['Python', 'statistics', 'probability', 'm_stats'],
  install_requires=[
          'numpy',
          'm_stats',
          'scipy',
          'matplotlib'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7'
  ],
)